/*
 * $Id: rb_edje.c 307 2005-03-23 17:30:30Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Edje.h>
#include <evas/rb_evas.h>
#include <evas/rb_evas_object.h>

#define __RB_EDJE_C
#include "rb_edje.h"
#include "rb_edje_main.h"
#include "rb_part.h"
#include "rb_messages.h"

VALUE cEdje;

static void c_mark (RbEdje *e)
{
	c_evas_object_mark (&e->real);

	if (!NIL_P (e->parts))
		rb_gc_mark (e->parts);

	if (!NIL_P (e->callbacks))
		rb_gc_mark (e->callbacks);

	if (!NIL_P (e->on_text_changed_cb))
		rb_gc_mark (e->on_text_changed_cb);
}

static void c_free (RbEdje *e)
{
	c_evas_object_free (&e->real, false);
	free (e);

	edje_shutdown ();
}

/*
 * call-seq:
 *  Edje::Edje.new(evas) => edje
 *
 * Creates an Edje::Edje object.
 */
static VALUE c_new (VALUE klass, VALUE evas)
{
	VALUE self, argv[1];
	RbEdje *edje = NULL;

	CHECK_CLASS (evas, cEvas);
	GET_OBJ (evas, RbEvas, e);

	edje_init ();

	self = Data_Make_Struct (klass, RbEdje, c_mark, c_free, edje);

	edje->real.real = edje_object_add (e->real);
	edje->parts = Qnil;
	edje->callbacks = Qnil;
	edje->on_text_changed_cb = Qnil;

	argv[0] = evas;
	rb_obj_call_init (self, 1, argv);

	return self;
}

/*
 * call-seq:
 *  edje.freeze => nil
 *
 * Freezes <i>edje</i>.
 */
static VALUE c_freeze (VALUE self)
{
	GET_OBJ (self, RbEdje, e);

	edje_object_freeze (e->real.real);

	return Qnil;
}

/*
 * call-seq:
 *  edje.thaw => nil
 *
 * Thaws <i>edje</i>.
 */
static VALUE c_thaw (VALUE self)
{
	GET_OBJ (self, RbEdje, e);

	edje_object_thaw (e->real.real);

	return Qnil;
}

/*
 * call-seq:
 *  edje.load(eet, group) => nil
 *
 * Loads <i>eet</i> into <i>edje</i>. <i>group</i> is the
 * name of the group to be displayed.
 */
static VALUE c_load (VALUE self, VALUE eet, VALUE group)
{
	GET_OBJ (self, RbEdje, e);

	Check_Type (eet, T_STRING);
	Check_Type (group, T_STRING);

	if (!edje_object_file_set (e->real.real, StringValuePtr (eet),
	                           StringValuePtr (group)))
		rb_raise (rb_eException, "Cannot load eet");

	return Qnil;
}

/*
 * call-seq:
 *  edje.get_size_min => array
 *
 * Returns an array that contains the minimum size
 * of <i>edje</i>.
 */
static VALUE c_get_size_min (VALUE self)
{
	int w = 0, h = 0;

	GET_OBJ (self, RbEdje, e);

	edje_object_size_min_get (e->real.real, &w, &h);

	return rb_ary_new3 (2, INT2FIX (w), INT2FIX (h));
}

/*
 * call-seq:
 *  edje.get_size_max => array
 *
 * Returns an array that contains the maximum size
 * of <i>edje</i>.
 */
static VALUE c_get_size_max (VALUE self)
{
	int w = 0, h = 0;

	GET_OBJ (self, RbEdje, e);

	edje_object_size_max_get (e->real.real, &w, &h);

	return rb_ary_new3 (2, INT2FIX (w), INT2FIX (h));
}

/*
 * call-seq:
 *  edje.part_exists?(part) => true or false
 *
 * Returns true if <i>edje</i> has a part called <i>part</i>,
 * else returns false.
 */
static VALUE c_part_exists_get (VALUE self, VALUE name)
{
	int r;

	GET_OBJ (self, RbEdje, e);

	Check_Type (name, T_STRING);

	r = edje_object_part_exists (e->real.real, StringValuePtr (name));

	return r ? Qtrue : Qfalse;
}

/*
 * call-seq:
 *  edje.part(part_name) => part
 *
 * Returns the <code>Edje::Part</code> object that corresponds to
 * <i>part_name</i>. If there's no part with that name in <i>edje</i>,
 * an exception is raised.
 */
static VALUE c_part_get (VALUE self, VALUE name)
{
	VALUE part;

	GET_OBJ (self, RbEdje, e);
	Check_Type (name, T_STRING);

	if (!edje_object_part_exists (e->real.real, StringValuePtr (name))) {
		rb_raise (rb_eException, "Unknown part name");
		return Qnil;
	}

	if (NIL_P (e->parts))
		e->parts = rb_hash_new ();

	if (NIL_P (part = rb_hash_aref (e->parts, name))) {
		part = TO_PART (self, name);
		rb_hash_aset (e->parts, name, part);
	}

	return part;
}

static void on_text_changed (void *data, Evas_Object *eo,
                             const char *part_name)
{
	VALUE self = (VALUE) data, part, name;

	GET_OBJ (self, RbEdje, e);

	name = rb_str_new2 (part_name);

	if (NIL_P (e->parts))
		e->parts = rb_hash_new ();

	if (NIL_P (part = rb_hash_aref (e->parts, name))) {
		part = TO_PART (self, name);
		rb_hash_aset (e->parts, name, part);
	}

	rb_funcall (e->on_text_changed_cb,
	            rb_intern ("call"), 1, part);
}

/*
 * call-seq:
 *  edje.on_text_changed { |part_obj| block }
 *
 * Registers a callback that will get called when the text
 * of any part is changed in <i>edje</i>.
 * The block is passed the <code>Edje::Part</code> object
 * of which the text changed.
 */
static VALUE c_on_text_changed (VALUE self)
{
	GET_OBJ (self, RbEdje, e);

	if (!rb_block_given_p ())
		return Qnil;

	e->on_text_changed_cb = rb_block_proc ();

	edje_object_text_change_cb_set (e->real.real, on_text_changed,
	                                (void *) self);

	return Qnil;
}

/*
 * call-seq:
 *  edje.emit_signal(signal, source) => nil
 *
 * Emits a signal to <i>edje</i>.
 *
 *  edje.emit_signal("signal_foo", "part_bar") #=> nil
 */
static VALUE c_emit_signal (VALUE self, VALUE signal, VALUE source)
{
	GET_OBJ (self, RbEdje, e);

	Check_Type (signal, T_STRING);
	Check_Type (source, T_STRING);

	edje_object_signal_emit (e->real.real, StringValuePtr (signal),
	                         StringValuePtr (source));

	return Qnil;
}

static void on_signal (void *data, Evas_Object *o,
                       const char *signal, const char *src)
{
	rb_funcall ((VALUE) data, rb_intern ("call"), 2,
	            rb_str_new2 (signal), rb_str_new2 (src));
}

/*
 * call-seq:
 *  edje.on_signal(signal [, source]) { |signal, source| block } => nil
 *
 * Registers a callback that will get called when <i>signal</i>
 * is emitted by <i>source</i>.
 * If source is nil, "*" will be used instead.
 * The block is passed two strings, signal and source, which identify
 * the emission.
 */
static VALUE c_on_signal (int argc, VALUE *argv, VALUE self)
{
	VALUE signal, src, cb;
	char *ssrc = "*";

	GET_OBJ (self, RbEdje, e);

	rb_scan_args (argc, argv, "11", &signal, &src);

	Check_Type (signal, T_STRING);

	if (!NIL_P (src)) {
		Check_Type (src, T_STRING);
		ssrc = StringValuePtr (src);
	}

	if (!rb_block_given_p ())
		return Qnil;

	cb = rb_block_proc ();

	if (NIL_P (e->callbacks))
		e->callbacks = rb_ary_new ();

	rb_ary_push (e->callbacks, cb);

	edje_object_signal_callback_add (e->real.real,
	                                 StringValuePtr (signal),
	                                 ssrc, on_signal, (void *) cb);

	return Qnil;
}

/*
 * call-seq:
 *  edje.play? => true or false
 *
 * Returns true if <i>edje</i> is in play mode, else returns false.
 */
static VALUE c_play_get (VALUE self)
{
	GET_OBJ (self, RbEdje, e);

	return edje_object_play_get (e->real.real) ? Qtrue : Qfalse;
}

/*
 * call-seq:
 *  edje.play(true or false)
 *
 * Sets <i>edje</i> to play resp. pause mode.
 */
static VALUE c_play_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEdje, e);

	CHECK_BOOL(val);

	edje_object_play_set (e->real.real, val == Qtrue);

	return Qnil;
}

/*
 * call-seq:
 *  edje.animation? => true or false
 *
 * Returns the animation state of <i>edje</i>.
 */
static VALUE c_animation_get (VALUE self)
{
	GET_OBJ (self, RbEdje, e);

	return edje_object_animation_get (e->real.real) ? Qtrue : Qfalse;
}

/*
 * call-seq:
 *  edje.animation(true or false)
 *
 * Sets the animation state of <i>edje</i>.
 */
static VALUE c_animation_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEdje, e);

	CHECK_BOOL(val);

	edje_object_animation_set (e->real.real, val == Qtrue);

	return Qnil;
}

static VALUE c_data_get (VALUE self, VALUE key)
{
	const char *s;

	GET_OBJ (self, RbEdje, e);

	Check_Type (key, T_STRING);

	s = edje_object_data_get (e->real.real, StringValuePtr (key));

	return s ? rb_str_new2 (s) : Qnil;
}

static VALUE c_send_message (VALUE self, VALUE msg)
{
	VALUE m;
	Edje_Message_Type type;
	int id;

	GET_OBJ (self, RbEdje, e);

	CHECK_CLASS (msg, cMsg);

	type = NUM2INT (rb_iv_get (msg, "@type"));
	id = NUM2INT (rb_iv_get (msg, "@id"));
	m = rb_funcall (msg, rb_intern ("serialize"), 0, NULL);

	edje_object_message_send (e->real.real, type, id, (void *) m);

	return Qnil;
}

void Init_Edje (void)
{
	cEdje = rb_define_class_under (mEdje, "Edje", cEvasObject);

	rb_define_singleton_method (cEdje, "new", c_new, 1);
	rb_define_method (cEdje, "freeze", c_freeze, 0);
	rb_define_method (cEdje, "thaw", c_thaw, 0);
	rb_define_method (cEdje, "load", c_load, 2);
	rb_define_method (cEdje, "get_size_min", c_get_size_min, 0);
	rb_define_method (cEdje, "get_size_max", c_get_size_max, 0);
	rb_define_method (cEdje, "part_exists?", c_part_exists_get, 1);
	rb_define_method (cEdje, "part", c_part_get, 1);
	rb_define_method (cEdje, "on_text_changed", c_on_text_changed, 0);
	rb_define_method (cEdje, "emit_signal", c_emit_signal, 2);
	rb_define_method (cEdje, "on_signal", c_on_signal, -1);
	rb_define_method (cEdje, "play?", c_play_get, 0);
	rb_define_method (cEdje, "play=", c_play_set, 1);
	rb_define_method (cEdje, "animation?", c_animation_get, 0);
	rb_define_method (cEdje, "animation=", c_animation_set, 1);
	rb_define_method (cEdje, "data", c_data_get, 1);
	rb_define_method (cEdje, "send_message", c_send_message, 1);
}
